#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/dsl.sh - configuration generator script for DSL (PPPOE)
#
# Creation:     2002-06-20  fm
# Last Update:  $Id: dsl.sh 77706 2020-05-28 15:49:23Z hbfl $
#
# Copyright (c) 2002-2004 Frank Meyer frank(at)eisfair(dot)org
# Copyright (c) 2010-2020 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# set package name
package_name=dsl

# include eislib
. /var/install/include/eislib

# include config
. /etc/config.d/${package_name}

VERSION=3.0.0

# ---------------------------------------------------------------------------
# check for name if udev provides
# ---------------------------------------------------------------------------
check_udev_dev_name()
{
   net_device=${1}

   echo "${net_device}" | /usr/bin/grep -Ewq '(enp[1-9]?[0-9]s[1-9]?[0-9][fu]?[1-9]?[0-9]?u?[1-9]?[0-9]?)|(en[os][1-9]?[0-9])|(enx[[:alnum:]]{12})'

   return ${?:-1}
}

# ---------------------------------------------------------------------------
# start DSL
# ---------------------------------------------------------------------------
dsl_start()
{
    cd /etc/rc2.d
    rm -f S??dsl
    ln -sf ../init.d/dsl S37dsl

    echo '"'"${DSL_PPPOE_USER}"'"  *  "'"${DSL_PPPOE_PASS}"'"' > /etc/ppp/pap-secrets
    echo '"'"${DSL_PPPOE_USER}"'"  *  "'"${DSL_PPPOE_PASS}"'"' > /etc/ppp/chap-secrets

    chmod 0700 /etc/ppp/pap-secrets
    chmod 0700 /etc/ppp/chap-secrets
}

# ---------------------------------------------------------------------------
# stop DSL
# ---------------------------------------------------------------------------
dsl_stop()
{
    rm -f /etc/rc2.d/S??dsl

    rm -f /etc/ppp/pap-secrets
    rm -f /etc/ppp/chap-secrets

    remove_udev_rule
}

# ---------------------------------------------------------------------------
# check for kernel version
# ---------------------------------------------------------------------------
check_for_sync()
{
    _eiskernel_version='1.22.0'
    if [ "${DSL_PPPOE_TYPE}" = "sync" ]
    then
        installed_version=$(eisman check 'eiskernel' "${_eiskernel_version}")

        case "${installed_version}" in
        new)
            change_parameter -n 'DSL_PPPOE_TYPE' -e 'async'

            mecho -n --info "For "
            mecho -n "'DSL_PPPOE_TYPE' 'sync' "
            mecho --info "please update your eiskernel first."
            echo
            anykey
            ;;
        *)
            : # nothing to do
            ;;
        esac
    fi
}

# ---------------------------------------------------------------------------
# remove udev rule
# ---------------------------------------------------------------------------
remove_udev_rule()
{
    udev_folder='/etc/udev/rules.d'
    udev_file=${udev_folder}/70-persistent-dsl.rules
    mkdir -p ${udev_folder}

    rm -f "${udev_file}"
}

# ---------------------------------------------------------------------------
# create udev rule
# ---------------------------------------------------------------------------
create_udev_rule()
{
    remove_udev_rule

    name=${DSL_PPPOE_NET}
    mac=${DSL_PPPOE_NET_MACADDR}

    if ! check_udev_dev_name ${name}
    then
        udev_require=true
    fi

    # check for net0:0, if so, for this networkdevice
    # no udev rule was written
    if echo "${name}" | grep -q ':'
    then
        udev_require=false
    fi

    if ${udev_require:-true}
    then
        con_n=$(lspci -nn | grep -c 'Ethernet controller')
        # if mac empty and >=2 networkdevices exists,
        # the mac address is required to wrote a correct udev rule
        # con_n = lspci -nn | grep -c 'Ethernet controller'
        if [ -z "${mac}" ] && [ ${con_n:-0} -ge 2 ]
        then
            echo
            mecho --warn  "Empty 'DSL_PPPOE_NET_MACADDR'"
            mecho --error 'More than one network card found;'
            mecho --error 'please enter the mac address!'
            echo
            udev_require=false
        fi
    fi

    if ${udev_require:-true}
    then
        echo 'SUBSYSTEM=="net", ACTION=="add", DRIVERS=="?*", ATTR{address}=="'${mac}'", ATTR{dev_id}=="0x0", ATTR{type}=="1", NAME="'${name}'"' >>${udev_file}
    fi
}

# ---------------------------------------------------------------------------
# set parameter to no
# ---------------------------------------------------------------------------
change_parameter()
{
    # rewrite prarameter
    _param_name=''
    _param_entry=''
    _param_file="/etc/config.d/${package_name}"

    while [ ${#} -gt 0 ]
    do
        case "${1}" in
        -n)
            _param_name="${2}"
            shift 2
            ;;
        -e)
            _param_entry="${2}"
            shift 2
            ;;
        esac
    done

    sed -i "s#^\(${_param_name}=['\"]\).*\(['\"]\)#\1${_param_entry}\2#" \
         ${_param_file}
}

# ---------------------------------------------------------------------------
# main
# ---------------------------------------------------------------------------
main()
{
    case ${1} in
    start)
        check_for_sync
        dsl_start
        create_udev_rule
        ;;
    stop)
        dsl_stop
        ;;
    *)
        case "${START_DSL}" in
        yes)
            check_for_sync
            dsl_start
            create_udev_rule
            ;;
        no)
            dsl_stop
            ;;
        esac
        ;;
    esac

    exit 0
}

# ---------------------------------------------------------------------------
# call function main
# ---------------------------------------------------------------------------
main "${@}"

# ---------------------------------------------------------------------------
# end
#----------------------------------------------------------------------------
